<?php
/*
Plugin Name: GDPress
Plugin URI: https://gdpress.eu
Description: More than a privacy plugin

Version: 2.0.1

Author: Andre Renaut
Author URI: https://blog.mailpress.org/contact/
License: WTFPL license

Text Domain: GDPress
Domain Path: /gp-content/languages
*/
//if ( !defined( 'GDP_DEBUG' ) ) define ( 'GDP_DEBUG', ( defined( 'WP_DEBUG' ) && ( WP_DEBUG ) ) );

/** GDPress GDPR Wordpress last date review. */
define ( 'GDP_GDPR_Review', '2020-05-25' );

/** Absolute path to the GDPress directory. */
define ( 'GDP_ABSPATH',	__DIR__ . '/' );

/** Folder name of GDPress plugin. */
define ( 'GDP_FOLDER', 	basename( GDP_ABSPATH ) );

/** Relative path to the GDPress directory. */
define ( 'GDP_PATH',    PLUGINDIR . '/' . GDP_FOLDER . '/' );

/** for admin plugin pages */
define ( 'GDPress_page_dashboard',	'gdpress_dashboard' );
define ( 'GDPress_page_ropa',		'gdpress_ropa' );
define ( 'GDPress_page_events',		'gdpress_events' );
define ( 'GDPress_page_event',		GDPress_page_events . '&file=event' );
define ( 'GDPress_page_settings',	'gdpress_settings' );

/** for admin plugin urls */
$gdp_file = 'admin.php';

define ( 'GDPress_dashboard',		add_query_arg( 'page', GDPress_page_dashboard, $gdp_file ) );
define ( 'GDPress_ropa',		add_query_arg( 'page', GDPress_page_ropa,      $gdp_file ) );
define ( 'GDPress_events',		add_query_arg( 'page', GDPress_page_events,    $gdp_file ) );
define ( 'GDPress_event',		add_query_arg( 'page', GDPress_page_event,     $gdp_file ) );
define ( 'GDPress_settings',		add_query_arg( 'page', GDPress_page_settings, 'options-general.php' ) );

/** for mysql */
global $wpdb;
$wpdb->gdp_events     = $wpdb->prefix . 'gdpress_events';
$wpdb->gdp_eventmeta  = $wpdb->prefix . 'gdpress_eventmeta';

class GDPress
{
	const option_name_general     = 'GDPress_general';

	const option_name_apis        = 'GDPress_apis';
	const option_name_dashboard   = 'GDPress_dashboard';
	const option_name_embed       = 'GDPress_embed';
	const option_name_oembed      = 'GDPress_oembed';
	const option_name_site_health = 'GDPress_site_health';
	const option_name_headers     = 'GDPress_headers';

	public static $const = array();

	function __construct() 
	{
		require_once( 'gp-load.php' );
		require_once( GDP_ABSPATH . 'gp-includes/gdpress-template.php' );
		self::const_();

		spl_autoload_register( array( __CLASS__, 'autoload' ) );					// for class loader

		global $gdp_general;
		$gdp_general = get_option( self::option_name_general );

		add_action( 'plugins_loaded',			array( __CLASS__, 'plugins_loaded' ) );		// for gettext
		add_action( 'init',				array( __CLASS__, 'init' ) );			// for init
		add_action( 'widgets_init',			array( __CLASS__, 'widgets_init' ) );		// for widget
 		add_action( 'gdpress_check_flood',		array( __CLASS__, 'check_flood' ), 10, 3 );	// for request

		add_action( 'wp_ajax_gdpress_form',		array( 'GDP_WP_Ajax', 'getInstance' ) );	// for ajax
		add_action( 'wp_ajax_nopriv_gdpress_form',	array( 'GDP_WP_Ajax', 'getInstance' ) );	// for ajax
		add_action( 'wp_ajax_gdp_ajax',			array( 'GDP_WP_Ajax', 'getInstance' ) );	// for ajax

		add_action( 'gdp_schedule', 			array( __CLASS__, 'gdp_schedule' ) );		// for RoPA
		add_action( 'gdp_install', 			array( __CLASS__, 'gdp_install' ) );		// for RoPA

		if ( $gdp_general )
		{
			add_action( 'upgrader_process_complete', 	array( __CLASS__, 'upgraded' ), 15, 2 );	// for RoPA
			add_action( 'switch_theme', 			array( __CLASS__, 'switch_theme' ), 15, 3 );	// for RoPA
			add_action( 'activate_plugin', 			array( __CLASS__, 'activate_plugin' ), 15, 2 ); // for RoPA
			add_action( 'deactivated_plugin',		array( __CLASS__, 'deactivated_plugin' ), 15,2);// for RoPA
		}

		if ( is_admin() )
		{
			register_activation_hook( plugin_basename( __FILE__ ), 	array( __CLASS__, 'install' ) );// for install
			register_deactivation_hook( plugin_basename( __FILE__ ), array( __CLASS__, 'uninstall' ) ); 

			add_action( 'admin_init',		array( __CLASS__, 'admin_init' ) );		// for admin css
			add_action( 'admin_menu',		array( __CLASS__, 'admin_menu' ) );		// for menu

		}
		add_action( 'wp_head',				array( __CLASS__, 'wp_head' ) );		// for admin bar on site

		add_shortcode( 'gdpress',			array( __CLASS__, 'shortcode' ) );		// for shortcode

	// to apply settings

	// to set mail headers if required
		$headers_option = get_option( self::option_name_headers );
		if ( isset( $headers_option['headers'] ) ) new GDP_Mail_Headers();

	// to archive events of the data subject related to export and erasure requests (for legal purpose ONLY)
		if ( isset( $gdp_general['archive'] ) ) GDP_Archivist::getInstance();

	// for API's
		$apis = get_option( self::option_name_apis);
		if ( isset( $apis['events_news'] ) ) 	new GDP_WP_off_Events_News();
		if ( isset( $apis['emojis'] ) ) 	new GDP_WP_off_Emojis();
		if ( isset( $apis['avatars'] ) )	new GDP_WP_off_Avatars();
		if ( isset( $apis['heartbeat'])) 	new GDP_WP_off_HeartBeat(); 	
		if ( isset( $apis['xmlrpc'] ) ) 	new GDP_WP_off_Xmlrpc();
		if ( isset( $apis['rest-api'] ) ) 	new GDP_WP_off_Rest_Api();

	// for Embed handlers
		$embed = get_option( self::option_name_embed);
		GDP_WP_Embed_handlers::getInstance();
		if ( $embed ) 				new GDP_WP_off_Embed();

	// for oEmbed providers
		$oembed = get_option( self::option_name_oembed);
		GDP_WP_oEmbed_providers::getInstance();
		if ( $oembed ) 				new GDP_WP_off_oEmbed();

	// for Site Health
		$site_health = get_option( self::option_name_site_health);
		GDP_WP_Site_Health_tests::getInstance();
		if ( $site_health ) 			new GDP_WP_off_Site_Health();


		do_action( 'GDPress_init' );
	}

	public static function const_()
	{
		self::$const = array (
			'export' => array(
				'post_name' => 'export_personal_data',
				'cap'  => 'export_others_personal_data',
				'desc' => __( 'Export Personal Data', 'GDPress' ),
				'short'=> __( 'export', 'GDPress' ),
				'menu' => array( 
					'css' => 'super',
					'mb'  => __( 'Pending Privacy Export', 'GDPress' ),
					'url' => admin_url( 'export-personal-data.php' ),
				),
			),
			'remove' => array(
				'post_name' => 'remove_personal_data',
				'cap'  => 'erase_others_personal_data',
				'desc' => __( 'Erase Personal Data', 'GDPress' ),
				'short'=> __( 'erase', 'GDPress' ),
				'menu' => array( 
					'css' => 'sub',
					'mb'  => __( 'Pending Privacy Erase', 'GDPress' ),
					'url' => admin_url( 'erase-personal-data.php' ),
				),
			),
		);
	}

	public static function autoload( $class )
	{
		if ( 0 == strpos( $class, 'GDP_' ) )
		{
			$file = GDP_ABSPATH . "gp-includes/class/{$class}.class.php";
			if ( is_file( $file ) ) return require $file;
		}
		return false;
	}

	public static function plugins_loaded() 
	{
		load_plugin_textdomain( 'GDPress', false, GDP_FOLDER . '/' . GDP_CONTENT_FOLDER . '/' . 'languages' );
	}

	public static function init()
	{
		global $gdp_general;

	// for roles & capabilities
		$role = get_role( 'administrator' );
		foreach ( self::capabilities() as $capability => $v )
		{
			$role->add_cap( $capability );
		}

		if ( GDP_WP_Role_COP::is_only() ) add_action( 'admin_menu', array( 'GDP_WP_Role_COP', 'admin_menu' ), 999 );

		do_action( 'GDPress_roles_and_capabilities' );

	// for admin bar menu
		add_action( 'admin_bar_menu', array( 'GDP_WP_Admin_Bar_Menu', 'getInstance' ), 61 );

	// for specific gdpress admin page
		if ( is_admin() && self::get_admin_page() )
		{
			self::admin_page();
		}
	}

	public static function capabilities()										// for roles & capabilities
	{
		include ( GDP_ABSPATH . 'gp-admin/includes/capabilities/capabilities.php' );
		return apply_filters( 'GDPress_capabilities', $capabilities );
	}

	public static function capability_groups()
	{
		include ( GDP_ABSPATH . 'gp-admin/includes/capabilities/capability_groups.php' );
		return apply_filters( 'GDPress_capability_groups', $capability_groups );
	}

	public static function widgets_init()
	{
		register_widget( 'GDP_Widget' );
	}

	public static function check_flood()
	{
		add_filter( 'gdpress_is_flood',     array( 'GDP_Request', 'is_flood'       ), 10, 4 );
		add_filter( 'gdpress_flood_filter', array( 'GDP_Request', 'throttle_flood' ), 10, 3 );
	}

//// ADMIN ////

	static function is_bot( $HTTP_USER_AGENT = NULL )
	{
		$bots_useragent = array( 'googlebot', 'google', 'bingbot', 'DotBot', '/bot.html', 'msnbot', 'RU_Bot', 'ia_archiver', 'lycos', 'jeeves', 'scooter', 'fast-webcrawler', 'slurp@inktomi', 'turnitinbot', 'technorati', 'yahoo', 'findexa', 'findlinks', 'gaisbo', 'zyborg', 'surveybot', 'bloglines', 'blogsearch', 'ubsub', 'syndic8', 'userland', 'gigabot', 'become.com' );
		$HTTP_USER_AGENT = $HTTP_USER_AGENT ?? filter_input( INPUT_SERVER, 'HTTP_USER_AGENT' );
		foreach ( $bots_useragent as $bot ) if ( stristr( $HTTP_USER_AGENT, $bot ) !== false ) return true;
		return false;
	}

// for install

	public static function install() 
	{
		$min_ver_wp = '5.4';
		include ( GDP_ABSPATH . 'gp-admin/includes/install/gdpress.php' );

		wp_unschedule_hook( 'gdp_schedule_weekly' );

		$now4cron = time();
		wp_schedule_single_event( $now4cron + 1, 'gdp_install', array( 'args' => array( 'event' => '** Install **' ) ) );

		$date = new DateTime( 'next sunday 23:00', wp_timezone() );
		$weekly = $date->getTimestamp();
		wp_schedule_event( $weekly, 'weekly', 'gdp_schedule', array( 'args' => array( 'event' => '** Weekly **' ) ) );

		GDP_WP_Role_COP::add();
	}

	public static function gdp_install() 
	{
		GDP_RoPA::do_all( 'gdpress_debut' );
	}

	public static function gdp_schedule() 
	{
		GDP_RoPA::do_all();
	}

	public static function uninstall() 
	{
		wp_unschedule_hook( 'gdp_install' );
		wp_unschedule_hook( 'gdp_schedule' );

		GDP_RoPA::do_all( 'gdpress_finale' );
	}

	public static function upgraded( $upgraded, $args ) 
	{
		$actions = array( 'core' => 'core', 'plugin' => 'plugins', 'theme' => 'themes' );
		if ( !isset($actions[$args['type']]) ) return;

		GDP_RoPA::do_action( $actions[$args['type']], 'upgraded' );		
	}

	public static function switch_theme( $new_name, $new_theme, $old_theme ) 
	{
		GDP_RoPA::do_action( 'themes', 'switched' );		
	}

	public static function activate_plugin( $id, $is_mu ) 
	{
		GDP_RoPA::do_action( 'plugin', 'activated', $id );		
	}

	public static function deactivated_plugin( $id, $is_mu ) 
	{
		GDP_RoPA::do_action( 'plugin', 'deactivated', $id );		
	}



// for admin stuff

	public static function admin_init()
	{
		wp_register_style( 'gdPress_admenu', '/' . GDP_PATH . 'gp-admin/css/admin-bar.css' );
		wp_enqueue_style ( 'gdPress_admenu' );
	}

// for menus and admin bar

	public static function wp_head()
	{
		echo '<link rel="stylesheet" id="gdPress_admenu-css" href="' . plugins_url() . '/gdpress/gp-admin/css/admin-bar.css" type="text/css" media="all" />';
	}

	public static function admin_menu()
	{
		new GDP_WP_Admin_Menu();
	}

// for admin page

	public static function get_admin_page()
	{
		$admin_page = filter_input( INPUT_GET, 'page' );
		$file       = filter_input( INPUT_GET, 'file' );

		return ( !isset( $admin_page ) || strpos( $admin_page, 'gdpress' ) !== 0 ) ? false : ( $admin_page . ( ( isset( $file ) ) ? '&file=' . $file : '' ) );
	}

	public static function admin_page()
	{
		$admin_page = self::get_admin_page();

		if ( !$admin_page ) return;

		$hub = array ( 	
				GDPress_page_dashboard	=> 'dashboard', 
				GDPress_page_ropa 	=> 'ropa', 
				GDPress_page_events 	=> 'events',
 				GDPress_page_event 	=> 'event',
				GDPress_page_settings 	=> 'settings', 
		);

		$hub = apply_filters( 'GDPress_load_admin_page', $hub );
		if ( !isset( $hub[$admin_page] ) )
		{
			return;
		}

		$file = GDP_ABSPATH . 'gp-admin/' . $hub[$admin_page] . '.php';
		if ( !is_file( $file ) )
		{
			return;
		}

		require_once( $file );
		if ( !class_exists( 'GDP_AdminPage' ) )
		{
			return;
		}

		new GDP_AdminPage();
	}

////	Request form	////

	public static function shortcode( $options = array() )
	{
		if (! is_array( $options ) ) unset ( $options );
		$options['widget_id'] = 'sc';

		ob_start();
			self::form( $options );
			$x = ob_get_contents();
		ob_end_clean();
		return $x; 
	}

	public static function form( $options = array() )
	{
		static $_widget_id = 0;
		$options['widget_id'] = ( isset( $options['widget_id'] ) ) ? $options['widget_id'] . '_' . $_widget_id : 'gdp_' . $_widget_id;
		GDP_Widget::widget_form( $options );
		$_widget_id++;
	}
}
new GDPress();