<?php
class GDP_AdminPage extends GDP_WP_Admin_page_list_
{
	const per_page = false;

	const screen		= 'gdpress_event';
	const capability	= 'GDPress_manage_events';
	const help_url		= '';
	const file			= __FILE__;

////  Title  ////

	public static function title()
	{
		global $title, $gdp_event;

		if ( isset( self::$get_['id'] ) )
		{
			$id = self::$get_['id'];
			$gdp_event = GDP_Event::get( $id );
		}
		if ( !$gdp_event ) wp_redirect( GDPress_events );

		$title = __( 'GDPress | Archive', 'GDPress' ); 
	}

//// Help ////

	public static function add_help_tab() 
	{
		global $current_screen;

		$content = '';
		$content .= '<p><strong>' . __( 'Archive :', 'GDPress' ) . '</strong></p>';

		$content .= '<p>' . __( 'This screen displays informations collected on one event related to the data subject and the WordPress privacy processes or a software component.', 'GDPress' ) . '</p>';

		$content .= '<p>' . __( 'These datas are kept under the authority of your Data Protection Officer for legal purpose ONLY.', 'GDPress' ) . '</p>';

		$content .= '<p>' . __( 'Personal events are retrieved to the data subject, but they cannot be deleted.', 'GDPress' ) . '</p>';

		$current_screen->add_help_tab( array( 	'id'		=> 'overview',
							'title'		=> __( 'Overview' ),
							'content'	=> $content )
		);
	}

////  Styles  ////

	public static function print_styles( $s ) 
	{
		global $gdp_event;

		$styles = ( is_array( $s ) ) ? $s : array();

		$styles[] = 'dashboard';

		if ( isset( $gdp_event->related ) && empty( $gdp_event->related ) )
		{
			wp_register_style( 'gdp_readme','/' . GDP_PATH . 'gp-admin/css/_install.css' );
			$styles[] = 'gdp_readme';
		}		

		if ( isset( $gdp_event->event ) && ( $gdp_event->event == 'exported' ) )
		{
			wp_register_style( 'gdp_export','/' . GDP_PATH . 'gp-admin/css/_export_html.css' );
			$styles[] = 'gdp_export';
		}		

		wp_register_style( 'gdp_event', 	'/' . GDP_PATH . 'gp-admin/css/events.css' );
		$styles[] = 'gdp_event';

		wp_register_style( 'gdp_ui',  '/' . GDP_PATH . 'gp-admin/css/_ui.css' );
		$styles[] = 'gdp_ui';

		wp_register_style( self::screen, 	'/' . GDP_PATH . 'gp-admin/css/event.css' );
		$styles[] = self::screen;

		parent::print_styles( $styles );
	}

////  Scripts  ////

	public static function print_scripts( $scripts = array() ) 
	{
		wp_register_script( self::screen, 		'/' . GDP_PATH . 'gp-admin/js/event.js', array( 'postbox', 'jquery-ui-tabs', ), false, 1 );
		wp_localize_script( self::screen, 		'GDP_AdminPageL10n',  array( 
			'screen' => self::screen,
		 ) );

		$scripts[] = self::screen;
		parent::print_scripts( $scripts );
	}

//// Columns ////

	public static function get_columns() 
	{
		global $gdp_event;

		$columns = array();
		$columns['title'] = __( 'Date', 'GDPress' );
		$columns['event']   = __( 'Event', 'GDPress' );

		if ( $gdp_event->family == '' )
			$columns['wp_user_id'] = __( 'Author', 'GDPress' );
		else
			$columns['related']    = __( 'Related', 'GDPress' );

		$columns['ip'] = __( 'Ip',  'GDPress'  );

		return $columns;
	}

////  Row  ////

	public static function get_row()
	{
		global $gdp_general, $gdp_event;

		$id = self::$get_['id'];

		$event = $gdp_event;

		$row_class = '';

		$out = '';
		$out .= '<tr id="event-' . $id . '" class="' . $row_class . '">';

		$columns = self::get_columns();
		$hidden  = self::get_hidden_columns();

		if ( isset( $columns['related'] ) && empty( $event->related ) ) $event->related = 'core';

		foreach ( $columns as $column_name => $column_display_name ) 
		{
			$attributes = sprintf( 'class="%1$s column-%1$s%2$s"', $column_name, ( in_array( $column_name, $hidden ) ) ? ' hidden' : '' );

 			switch ( $column_name ) 
			{
				case 'title' :
					$attributes = 'class="title column-title column-primary page-title"';
					$out .= '<td ' . $attributes . '>';
					$out .= '<strong>';
					$out .= $event->created;
					$out .= '</strong>';
					$out .= '</td>';
				break;
				case 'wp_user_id' :
					$out .= '<td ' . $attributes . '>';
					$wp_user = get_userdata( $event->wp_user_id );
					if ( $wp_user )
					{
						$out .= $wp_user->display_name;
					}
					elseif ( $event->wp_user_id )
					{
						$out .= sprintf( __( '(unknown[%1$s])', 'GDPress' ), $event->wp_user_id );
					}
					$out .= '</td>';
				break;
				case 'event' :
					$out .= '<td ' . $attributes . '>';
					$out .= ( $event->data_subject ) ? $event->event : $event->event . ' <sup>*</sup>';
					$out .= '</td>';
				break;
				default :
					$out .= '<td ' . $attributes . '>';
					$out .= $event->{$column_name};
					$out .= '</td>';
				break;
			}
		}
		$out .= '</tr>';

		return $out;
	}

////  Metaboxes  ////

	public static function admin_head() 
	{
		global $gdp_general, $gdp_event;

		$id = self::$get_['id'];

		$event = $gdp_event;

		$_items = array(
			'current_theme' => array( array( __( 'Stylesheet Data', 'GDPress' ), 'data', 'side', ), array( __( 'Readme', 'GDPress' ), 'readme', 'normal', ), ),
			'data_plugin'	=> array( array( __( 'Plugin Data', 'GDPress' ), 'data', 'normal', ), ),
			'data_mu-plugin'=> array( array( __( 'Plugin Data', 'GDPress' ), 'data', 'normal', ), ),
			'readme'  	=> array( array( __( 'Readme', 'GDPress' ), 'readme', 'side', ), ),
		);

		foreach( $event->info as $item => $content )
		{
			if ( ( !isset( $_items[$item] ) ) ) $item = str_replace ( '-', '_', $item . '_' . $event->family );
		
			if ( ( isset( $_items[$item] ) ) )
			{
				foreach( $_items[$item] as $_item )
				{
					if ( !method_exists( __CLASS__, 'meta_box_' . $_item[1] ) ) continue;
					add_meta_box( $item . '_' . $_item[1] , $_item[0], array( __CLASS__, 'meta_box_' . $_item[1] ), self::screen, $_item[2], 'core' );
				}
  			}
		}

/**/

		$_metas = array(
			'func'		  => array( __( 'Related Function Info', 'GDPress' ), 'normal', ),
			'message'	  => array( __( 'Mail build', 'GDPress' ), 'side', ),
			'request'	  => array( __( 'Request at Time of Event', 'GDPress' ), 'normal', ),
			'export_html'	  => array( __( 'Html Export', 'GDPress' ), 'side', ),
		);

		$metas = GDP_Event_meta::get( $id );

		if ( !is_array( $metas ) ) $metas = array( $metas );

		foreach( $metas as $meta )
		{
			if ( ( isset( $_metas[$meta->meta_key] ) ) && ( method_exists( __CLASS__, 'meta_box_' . $meta->meta_key ) ) ) 
				add_meta_box( $meta->meta_key, $_metas[$meta->meta_key][0], array( __CLASS__, 'meta_box_' . $meta->meta_key ), self::screen, $_metas[$meta->meta_key][1], 'core' );
			
		}

		if ( defined( 'GDP_DEBUG' ) && ( GDP_DEBUG ) )
		{
			add_meta_box( 'debug_backtrace', __( 'Debug_backtrace', 'GDPress' ), array( __CLASS__, 'meta_box_debug_backtrace' ), self::screen, 'side',   'core' );
			add_meta_box( 'Dump_info',       __( 'Dump info', 'GDPress' ),       array( __CLASS__, 'meta_box_dump_info' ),       self::screen, 'normal', 'core' );
		}

		do_action( 'GDPress_add_meta_boxes_event', $id, self::screen );

		parent::admin_head();
	}

/**/

	public static function meta_box_request( $event )
	{
		$request = GDP_Event_meta::get( $event->id, 'request' );

		$cols = array( 'ID', 'user_id', 'action_name', 'status', 'created_timestamp', 'modified_timestamp', 'confirmed_timestamp', 'completed_timestamp', 'request_data', );

		$out = '<table>';
		foreach( $cols as $col )
		{
			$value = '';
			switch( $col )
			{
				case 'user_id' :
					if ( !$request->{$col} ) continue 2;
					$wp_user = get_userdata( $request->{$col} );
					$value = $wp_user->display_name ?? sprintf( __( '(unknown[%1$s])', 'GDPress' ), $request->{$col} );
				break;
				case 'created_timestamp' :
				case 'modified_timestamp' :
				case 'confirmed_timestamp' :
				case 'completed_timestamp' :
					if ( !$request->{$col} ) continue 2;
					$value = wp_date( 'Y-m-d H:i:s', $request->{$col} ); 
				break;
				case 'request_data' :
					if ( empty( $request->{$col} ) ) continue 2;
					$value = is_array( $request->{$col} ) ? maybe_serialize( $request->{$col} ) : $request->{$col}; 
				break;
				default :
					$value = $request->{$col};
				break;
			}
			$out .= '<tr><td><b>' . $col . '</b></td><td>' . $value . '</td></tr>';
		}
		$out .= '</table>';
	
		echo $out;
	}

/**/

	public static function meta_box_export_html( $event )
	{
		$html = GDP_Event_meta::get( $event->id, 'export_html' );

		$out = '<div class="meta_box_content">';

		$html = substr( $html, strpos( $html, '<body' ) + 6 );
		$html = str_replace( array( '</body>', '</html>', ), '', $html );
		$out .= '<div id="export_html_content">' . $html . '</div>';

		$out .= '</div>';

		echo $out;
	}

/**/

	public static function meta_box_message( $event )
	{
		$message = GDP_Event_meta::get( $event->id, 'message' );

		$out = '<div class="meta_box_content">';

		$out .= '<pre>';
		$out .= htmlspecialchars( $message );
		$out .= '</pre>';

		$out .= '</div>';

		echo $out;
	}

/**/

	public static function meta_box_data( $event )
	{
		$out = '<div class="meta_box_content">';

		switch( true )
		{
			case ( isset( $event->info['parent']['data']['headers'] ) ) :
				$out .= '<div class="event-tabs">';
				$out .= '<ul class="ul-tabs"><li class="tabs"><a href="#tabs-data-0">' . $event->info['current']['data']['Name'] . '</a></li><li class="tabs"><a href="#tabs-data-1">' . $event->info['parent']['data']['Name'] . '</a></li></ul>';
				$out .= '<div id="tabs-data-0">' . self::meta_box_data_renderer( $event->info['current']['data']['headers'] ) . '</div>';				
				$out .= '<div id="tabs-data-1">' . self::meta_box_data_renderer( $event->info['parent']['data']['headers'] ) . '</div>';
				$out .= '</div>';
			break;
			case ( isset( $event->info['current']['data']['headers'] ) ) :
				$out .= self::meta_box_data_renderer( $event->info['current']['data']['headers'] );			
			break;
			default :
				$out .= self::meta_box_data_renderer( $event->info['data'] );
			break;
		}

		$out .= '</div>';
		echo $out;
	}

	public static function meta_box_data_renderer( $data )
	{
		$out = '<table>';
		foreach( $data as $k => $v )
		{
			switch( $k )
			{
				case 'RoPA_time' :
					$RoPA_time = $v;
					continue 2;
				break;
				default :
					$out .= '<tr><td><b>' . $k . '</b></td><td>' . $v . '</td></tr>';
				break;
			}
		}
		$out .= '</table>';
	
		return $out;
	}

/**/

	public static function meta_box_readme( $event )
	{
		$out = '<div class="meta_box_content">';

		switch( true )
		{
			case ( isset( $event->info['readme']['html'] ) ) :
				$readme = $event->info['readme']['html'];
				$readme = substr( $readme, strpos( $readme, '<body' ) + 6 );
				$readme = str_replace( array( '</body>', '</html>', ), '', $readme );
				$readme = str_replace( '"wp-admin/images', '"../wp-admin/images', $readme );
				$out .= '<div><div id="readme_html">' . $readme . '</div></div>';
			break;
			case ( isset( $event->info['parent']['readme'] ) ) :
				$out .= '<div class="event-tabs">';
				$out .= '<ul class="ul-tabs"><li class="tabs"><a href="#tabs-readme-0">' . $event->info['current']['data']['Name'] . '</a></li><li class="tabs"><a href="#tabs-readme-1">' . $event->info['parent']['data']['Name'] . '</a></li></ul>';
				$out .= '<div id="tabs-readme-0">' . self::meta_box_readme_renderer( $event->info['current']['readme'] ) . '</div>';				
				$out .= '<div id="tabs-readme-1">' . self::meta_box_readme_renderer( $event->info['parent']['readme'] ) . '</div>';
				$out .= '</div>';
			break;
			case ( isset( $event->info['current']['readme'] ) ) :
				$out .= self::meta_box_readme_renderer( $event->info['current']['readme'] );			
			break;
			default :
				$out .= self::meta_box_readme_renderer( $event->info['readme'] );
			break;
		}

		$out .= '</div>';

		echo $out;
	}

	public static function meta_box_readme_renderer( $readme )
	{
		if (! is_array( $readme ) ) return '<pre class="readme">' . __('> Unabled to access file /readme.txt', 'GDPress') . '</pre>';

		$out = '<pre class="readme">';
		foreach( $readme as $section => $content )
		{
			$lines = array();
			switch( $section )
			{
				case 'name' :
					$name = $content;
					continue 2;
				break;
				case 'RoPA_time' :
					$RoPA_time = $content;
					continue 2;
				break;
				default :
					if ( is_array( $content ) ) 
					{
						$lines[] = ( $section == $name ) ? "=== $section ===" :  "\r\n== $section ==";
						foreach( $content as $line ) { $lines[] = $line; }
					}
				break;
			}
			$out .= htmlspecialchars( implode( "\r\n", $lines ) );
		}
		$out .= '</pre>';
	
		return $out;
	}

/**/

	public static function meta_box_func( $event )
	{
		$func = GDP_Event_meta::get( $event->id, 'func' );

		echo '<div class="meta_box_content"><pre>';
		var_dump( $func );
		echo '</pre></div>';
	}


	public static function meta_box_dump_info( $event )
	{
		echo '<div class="meta_box_content"><pre>';
		var_dump( $event );
		echo '</pre></div>';
	}

	public static function meta_box_debug_backtrace( $event )
	{
		$dbt = GDP_Event_meta::get( $event->id, 'debug_backtrace' );

		$out = ( is_array( $dbt ) ) ? GDP_Event::format_debug_backtrace( $dbt ) : $dbt;

		echo '<div class="meta_box_content"><pre>' . $out . '</pre></div>';
	}
}