<?php
class GDP_AdminPage extends GDP_WP_Admin_page_list_
{
	const screen		= GDPress_page_events;
	const capability	= 'GDPress_manage_events';
	const help_url		= false;
	const file		= __FILE__;

////  Redirect  ////

	public static function redirect() 
	{
		$url_parms = self::get_url_parms( array( 's', 'paged', 'ip', 'author', ) );

		if ( isset( $url_parms['message'] ) ) return;

		if ( isset( $url_parms['s'] ) )
		{
			if ( is_email(   $url_parms['s'] ) ) return;
			if ( filter_var( $url_parms['s'], FILTER_VALIDATE_IP ) && ( isset( $url_parms['ip'] ) )     && ( $url_parms['s'] == $url_parms['ip']     ) ) return;
			if ( is_numeric( $url_parms['s'] )                     && ( isset( $url_parms['author'] ) ) && ( $url_parms['s'] == $url_parms['author'] ) ) return;

			if ( filter_var( $url_parms['s'], FILTER_VALIDATE_IP ) )
			{
				$url_parms['ip'] = $url_parms['s'];
			}
			elseif ( is_numeric( $url_parms['s'] ) )
			{
				$url_parms['author'] = $url_parms['s'];
			}
			else
			{
				$url_parms['message'] = $url_parms['s'];
			}
			unset( $url_parms['s'] );
			self::gdp_redirect( self::url( GDPress_events, $url_parms ) );
		}
	}

////  Title  ////

	public static function title() 
	{
		global $title; 
		$title = __( 'GDPress | Archives', 'GDPress' ); 
	}

//// Help ////

	public static function add_help_tab() 
	{
		global $current_screen;

		$content = '';
		$content .= '<p><strong>' . __( 'Archives :', 'GDPress' ) . '</strong></p>';

		$content .= '<p>' . __( 'This screen lists all events related to the data subject and the WordPress privacy processes.', 'GDPress' ) . '</p>';

		$content .= '<p>' . __( 'These datas are kept under the authority of your Data Protection Officer for legal purpose ONLY.', 'GDPress' );

		$content .=         __( ' As archives, they can be retrieved to the data subject, but not deleted.', 'GDPress' ) . '</p>';

		$content .=         __( ' Events with a small <b><sup>*</sup></b> are considered as internal events and are not part of the WP privacy export process.', 'GDPress' ) . '</p>';

		$current_screen->add_help_tab( array( 	'id'		=> 'overview',
							'title'		=> __( 'Overview' ),
							'content'	=> $content )
		);
	}

////  Styles  ////

	public static function print_styles( $s ) 
	{
		$styles = ( is_array( $s ) ) ? $s : array();

		wp_register_style( self::screen, '/' . GDP_PATH . 'gp-admin/css/events.css' );
		$styles[] = self::screen;

		parent::print_styles( $styles );
	}

//// Columns ////

	public static function get_columns() 
	{
		$columns = array(
			'title'	=> __( 'Date', 'GDPress' ),
			'event'		=> __( 'Event', 'GDPress' ),
			'wp_user_id'	=> __( 'Author', 'GDPress' ), 
			'ip'		=> __( 'Ip',  'GDPress'  ),
		);
		return $columns;
	}


//// List ////

	public static function get_list( $args ) 
	{
		extract( $args );

		global $wpdb;

		$where = $_where = ' AND a.family = \'\'';
		$order = 'a.created DESC, a.id DESC';

		if ( isset( $url_parms['s'] ) )
		{
			$sc = false;
			$search = $url_parms['s'];

			if ( is_email( $url_parms['s'] ) )
			{
				$sc[] = 'a.related';
				$search = md5( $url_parms['s'] );
			}
			else
			{
				if ( filter_var( $url_parms['s'], FILTER_VALIDATE_IP ) ) $sc[] = 'a.ip';

				if ( is_numeric( $url_parms['s'] ) ) $sc[] = 'a.wp_user_id';
			}

			if ( $sc ) $where .= self::get_search_clause( $search, $sc );
		}

		if ( isset( $url_parms['author'] ) && ( $url_parms['author'] ) && is_numeric( $url_parms['author'] ) )
		{
			$where .= " AND a.wp_user_id = {$url_parms['author']}";
		}

		if ( isset( $url_parms['ip'] ) && ( $url_parms['ip'] ) )
		{
			$where .= " AND a.ip = '{$url_parms['ip']}'";
		}

		$where_counts = $where;

		if ( isset( $url_parms['data_subject'] ) )
		{
			$where .= ( $url_parms['data_subject'] ) ? ' AND a.data_subject IS TRUE' : ' AND a.data_subject IS FALSE';
		}

		if ( $where == $_where ) $where .= ' AND 1 <> 1';

		$args['query'] = "SELECT DISTINCT SQL_CALC_FOUND_ROWS a.id, a.created, a.event, a.related, a.data_subject, a.wp_user_id, a.ip, a.agent, a.uri, a.info FROM $wpdb->gdp_events a WHERE 1=1 $where ORDER BY $order";
		$args['cache_name'] = 'gdp_event';

		list( $_events, $total ) = parent::get_list( $args );

		$counts = array();
		$query = "SELECT data_subject, count( * ) as count FROM $wpdb->gdp_events a WHERE 1=1 $where_counts GROUP BY data_subject ORDER BY data_subject;";
		$statuses = $wpdb->get_results( $query );

		$subsubsub_urls = false;

		$libs = array(	'search'=> __( 'Search Results', 'GDPress'  ),
				0	=> __( 'Internal', 'GDPress' ),
				1	=> __( 'Personal', 'GDPress' ),
		);

		$_args = array_intersect_key( $url_parms, array_flip( array( 's', 'ip', 'author' ) ) );
		if ( isset( $_args['s'] ) ) foreach( array( 'ip', 'author', ) as $_arg ) { if ( ( isset( $_args[$_arg] ) ) && ( $_args['s'] == $_args[$_arg] ) ) unset( $_args['s'] ); }

		if ( $statuses )
		{
			$counts['search'] = 0;
			foreach( $statuses as $status )
			{
				if ( $status->count )
				{
					$i = (int) $status->data_subject;
					$counts[$i] = $status->count;
					$counts['search'] += $status->count;
				}
			}

			$out = array();

			foreach( $libs as $k => $lib )
			{
 				if ( !isset( $counts[$k] ) || !$counts[$k] )
				{
					continue;
				}

				$args = $_args;

				if ( is_numeric( $k ) ) $args['data_subject'] = $k;
				$url = esc_url( add_query_arg( $args, GDPress_events ) );

				$cls = '';
				if ( isset( $url_parms['data_subject'] ) )
				{
					if ( $k == $url_parms['data_subject'] )
					{
						$cls = ' class="current"';
					}
				}
				elseif ( !is_numeric( $k ) )
				{
					$cls = ' class="current"';
				}

				$out[] = sprintf( '<a%1$s href="%2$s">%3$s <span class="count">( <span class="event-count-%4$s">%5$s</span> )</span></a>', $cls, $url, $lib, $k, $counts[$k] );
			}

			if ( !empty( $out ) )
			{
				$subsubsub_urls = '<li>' . join( ' | </li><li>', $out ) . '</li>';
			}
		}

		return array( $_events, $total, $subsubsub_urls );
	}

////  Row  ////

	public static function get_row( $id, $url_parms, $checkbox = true )
	{
		global $gdp_event;
		static $row_class = '';

		$gdp_event = $event = GDP_Event::get( $id );

// url's
		$args		= array();
		$args['id']	= $id;

		$view_url	= esc_url( self::url( GDPress_event, array_merge( $args, $url_parms ) ) );

		unset( $args['action'] );

// actions
		$actions = array();
		$actions['view'] = '<a href="' . $view_url . '" title="' . esc_attr( sprintf( __( 'View event #%1$s', 'GDPress' ), $id ) ) . '">' . __( 'View' ) . '</a>';

// author
		$author = $event->wp_user_id ?: '';
		if ( is_numeric( $author ) ) 
		{
			$wp_user 	= get_userdata( $author );
			$author_url 	= esc_url( self::url( add_query_arg( 'author', $author, GDPress_events ), $url_parms ) );
		}

// ip
		$ip = $event->ip;
		$ip_url	= esc_url( self::url( add_query_arg( 'ip', $ip, GDPress_events ), $url_parms ) );

// table row 
//	class
		$row_class = '';

		$out = '';
		$out .= '<tr id="event-' . $id . '" class="' . $row_class . '">';

		$columns = self::get_columns();
		$hidden  = self::get_hidden_columns();

		foreach ( $columns as $column_name => $column_display_name ) 
		{
			$attributes = sprintf( 'class="%1$s column-%1$s%2$s"', $column_name, ( in_array( $column_name, $hidden ) ) ? ' hidden' : '' );

 			switch ( $column_name ) 
			{
				case 'title' :
					$attributes = 'class="title column-title has-row-actions column-primary page-title"';
					$out .= '<td ' . $attributes . '>';
					$out .= '<strong>';
					$out .= '<a class="row-title" href="' . $view_url . '" title="' . esc_attr( sprintf( __( 'View event #%1$s', 'GDPress' ), $id ) ) . '">';
					$out .= $event->created;
					$out .= '</a>';
					$out .= '</strong>';
					$out .= self::get_actions( $actions );
					$out .= '</td>';
				break;
				case 'wp_user_id' :
					$out .= '<td ' . $attributes . '>';
					if ( is_numeric( $author ) )
					{
						$out .= '<a href="' . $author_url . '" title="' . esc_attr( sprintf( __( 'Events by &#8220;%1$s&#8221;', 'GDPress' ), $wp_user->display_name ) ) . '">' . $wp_user->display_name . '</a>';
					}
					elseif ( $author )
					{
						$out .= sprintf( __( '(unknown[%1$s])', 'GDPress' ), $event->wp_user_id );
					}
					$out .= '</td>';
				break;
				case 'ip' :
					$out .= '<td ' . $attributes . '>';
					$out .= '<a href="' . $ip_url . '" title="' . esc_attr( sprintf( __( 'Events by &#8220;%1$s&#8221;', 'GDPress' ), $ip ) ) . '">' . $ip . '</a>';
					$out .= '</td>';
				break;

				case 'event' :
					$out .= '<td ' . $attributes . '>';
					$out .= ( $event->data_subject ) ? $event->event : $event->event . ' <sup>*</sup>';
					$out .= '</td>';
				break;
				default :
					$out .= '<td ' . $attributes . '>';
					$out .= $event->{$column_name};
					$out .= '</td>';
				break;
			}
		}
		$out .= '</tr>';

		return $out;
	}
}