<?php
class GDP_AdminPage extends GDP_WP_Admin_page_list_
{
	const per_page = false;

	const screen		= GDPress_page_ropa;
	const capability	= 'GDPress_manage_ropa';
	const help_url		= false;
	const file		= __FILE__;

	const add_form_id	= 'add';
	const list_id		= 'the-list';
	const tr_prefix_id	= 'component';

	static $_theme;

	function __construct()
	{
		parent::__construct();

		// for templates
		add_action( 'admin_footer', array( &$this, 'admin_footer' ) );

		self::$_theme = wp_get_theme();
	}

////  Title  ////

	public static function title() 
	{
		global $title; 
		$title = __( 'GDPress | RoPA Assistant', 'GDPress' ); 
	}

//// Help ////

	public static function add_help_tab() 
	{
		global $current_screen;

		$content = '';
		$content .= '<p><strong>' . __( 'RoPA (Records of Processing Activities) Assistant', 'GDPress' ) . '</strong></p>';
		$content .= '<p>' . __( '<b>THIS SCREEN IS NOT THE RECORDS OF PROCESSING ACTIVITIES</b> (<code>GDPR Article 30</code>).<br />This screen is the result of collected informations to help you building it.<br />', 'GDPress' ) . '</p>';

		$content .= '<p>' . __( 'With WordPress, your installation can be categorized in three different parts : WordPress itself also called core, Plugins and Themes.<br />', 'GDPress' );
		$content .=         __( 'The collected informations are based on the documentation included or attached to the different parts of software that compose your installation. ', 'GDPress' ) . '<br />';
		$content .=         __( 'Any piece of software can use external services that could collect personal data, and can serve other pieces of software (javascript, css, svgs, &#8230;), fonts, images, videos, maps, etc &#8230; ', 'GDPress' ) . '</p>';
		$content .= '<p>' . __( 'GDPress archives the readme files to help you maintain a log of your processes.<br />', 'GDPress' );
		$content .=         __( 'GDPR section involved in this feature is : <code>GDPR Chapter IV - Controller and Processor, Section 1 - General obligations</code>.', 'GDPress' ) . '</p>';

		$current_screen->add_help_tab( array( 	'id'		=> 'overview',
							'title'		=> __( 'Overview' ),
							'content'	=> $content )
		);

		$content = '';
		$content .= '<p><strong>' . __( 'RoPA Assistant | WordPress', 'GDPress' ) . '</strong></p>';
		$content .= '<p>' . __( 'Definitely, <b>THE</b> most important part of your installation.<br />', 'GDPress' );

		$current_screen->add_help_tab( array( 	'id'		=> 'wordpress',
							'title'		=> __( 'WordPress' ),
							'content'	=> $content )
		);

		$content = '';
		$content .= '<p><strong>' . __( 'RoPA Assistant | Plugins', 'GDPress' ) . '</strong></p>';
		$content .= '<p>' . sprintf( __( 'The information collected is an extract of the <code>readme.txt</code> file and the main file of your plugins%1$s.', 'GDPress' ), ( is_multisite() ) ? __( ' (multi-site plugins included).', 'GDPress' ) : '' ) . '</p>';
		$content .= '<p>' . __( 'GDPress looks also for a section that is not yet a standard : "Privacy".', 'GDPress' ) . '</p>';
		$content .= '<p>' . __( 'You should see this section displayed for GDPress.', 'GDPress' ) . '</p>';

		$current_screen->add_help_tab( array( 	'id'		=> 'plugins',
							'title'		=> __( 'Plugins' ),
							'content'	=> $content )
		);

		$content = '';
		$content .= '<p><strong>' . __( 'RoPA Assistant | Themes', 'GDPress' ) . '</strong></p>';
		$content .= '<p>' . __( 'The information collected is an extract of the <code>readme.txt</code> file and the main file of your theme and eventually its parent theme.', 'GDPress' ) . '</p>';
		$content .= '<p>' . __( 'GDPress looks also for a section that is not yet a standard : "Privacy".', 'GDPress' ) . '</p>';

		$current_screen->add_help_tab( array( 	'id'		=> 'themes',
							'title'		=> __( 'Themes' ),
							'content'	=> $content )
		);
	}

////  Styles  ////

	public static function print_styles( $s ) 
	{
		$styles = ( is_array( $s ) ) ? $s : array();

		wp_register_style( 'gdp_ui',  '/' . GDP_PATH . 'gp-admin/css/_ui.css' );
		$styles[] = 'gdp_ui';

		wp_register_style( self::screen,  '/' . GDP_PATH . 'gp-admin/css/ropa.css' );
		$styles[] = self::screen;

		parent::print_styles( $styles );
	}


////  Scripts  ////

	public static function print_scripts( $scripts = array() )
	{
		wp_register_script( self::screen, 		'/' . GDP_PATH . 'gp-admin/js/ropa.js', array( 'jquery-ui-tabs', 'wp-util' ) , false, 1 );
		wp_localize_script( self::screen, 		'GDP_AdminPageL10n',  array( 'screen' => self::screen ) );

		$scripts[] = self::screen;
		parent::print_scripts( $scripts );
	}

//// Columns ////

	public static function get_columns() 
	{
		$columns = array(
			'title'	=> __( 'Name',    'GDPress' ),
			'ver'	=> __( 'Version', 'GDPress' ),
			'event'	=> __( 'last',   'GDPress' ),
			'count'	=> __( '#',   'GDPress' )
		);
		return $columns;
	}
//// List ////

	public static function get_list( $args = array() ) 
	{
		extract( $args );

		global $wpdb;

		$query = "SELECT DISTINCT SQL_CALC_FOUND_ROWS MAX(id) as id, family, related FROM $wpdb->gdp_events WHERE 1=1 AND family <> '' GROUP BY  family, related ORDER BY  family, related";

		$components = $wpdb->get_results( $query );

		$total = $wpdb->get_var( "SELECT FOUND_ROWS()" );

		$libs = array(	'all'		=> __( 'All' ), 
				'core'		=> __( 'WP', 'GDPress' ),
				'mu-plugin'	=> __( 'MU Plugin', 'GDPress' ),
				'plugin'	=> __( 'Plugin', 'GDPress' ),
				'theme'		=> __( 'Theme', 'GDPress' ),
		);

		$subsubsub_urls = '';

		if ( $components )
		{
			foreach( $components as $component )
			{
				$family = $component->family;
				if ( 'mu-plugin' == $family )  $family = 'plugin';                                
				if ( !isset( $counts[$component->family] ) ) $counts[$component->family] = 1;
				else		  ++$counts[$component->family];
			}

			$out = array();

			foreach( $libs as $k => $lib )
			{
				if ( !isset( $counts[$k] ) || !$counts[$k] )
				{
					continue;
				}

				$out[] = sprintf( '<a href="#" class="subsubsub_a%4$s" data-family="%1$s">%2$s <span class="count">(<span class="%2$s-count-all">%3$s</span>)</span></a>', $k, $lib, $counts[$k], ( 'core' == $k ) ? ' current' : '' );
			}

			if ( !empty( $out ) )
			{
				$subsubsub_urls = '<li>' . join( ' | </li><li>', $out ) . '</li>';
			}
		}

		return array( $components, $total, $subsubsub_urls );
	}


////  Row  ////

	public static function get_row( $component_id ) 
	{
		static $row_class = '';

		$component = GDP_Event::get( $component_id );

		$row_class  = ( 'alternate' == substr( $row_class, 0, 9 ) ) ? '' : 'alternate ';
		$row_class .= 'iedit ' . $component->family;

		switch( $component->family )
		{
			case 'theme' :
				$current_path = self::$_theme->theme_root . '/' . self::$_theme->stylesheet;
				$event_path   = $component->info['current']['data']['theme_root'] . '/' . $component->info['current']['data']['stylesheet'];
				$row_class .= ( $current_path != $event_path ) ? ' hidden disabled' : ' hidden';
			break;
			case 'mu-plugin' :
				$row_class .= ( !is_plugin_active_for_network( $component->related ) ) ? ' hidden disabled' : ' hidden';
			break;
			case 'plugin' :
				$row_class .= ( !is_plugin_active( $component->related ) ) ? ' hidden disabled' : ' hidden';
			break;
		}

		$id = str_replace( array( '\\', '/', '-' ), '_', $component->related );

		$out = '';
		$out .= '<tr id="' . self::tr_prefix_id . '-' . $id . '" class="' . $row_class . '" data-family="' . esc_attr( $component->family ) . '" data-related="' . esc_attr( $component->related ) . '">';

		$columns = self::get_columns();
		$hidden  = self::get_hidden_columns();

		foreach ( $columns as $column_name => $column_display_name ) 
		{
			$attributes = sprintf( 'class="%1$s column-%1$s%2$s"', $column_name, ( in_array( $column_name, $hidden ) ) ? ' hidden' : '' );

			switch ( $column_name ) 
			{
				case 'title' :
					$attributes = 'class="title column-title column-primary page-title"';
					$out .= '<td ' . $attributes . '>';
					$out .= '<strong>';
					$out .= '<a class="row-title" href="#" title="' . esc_attr( $component->related ) . '">';
					$out .= self::get_name( $component );
					$out .= '</a>';
					$out .= '</strong>';
					$out .= '</td>';
				break;
	 			break;
	 			case 'desc':
	 				$out .= '<td ' . $attributes . '>' . self::get_desc( $component ) . '</td>';
	 			break;
	 			case 'ver':
	 				$out .= '<td ' . $attributes . '>' . self::get_ver( $component ) . '</td>';
	 			break;
	 			case 'event':
	 				$out .= '<td ' . $attributes . '>' . self::get_event( $component ) . '</td>';
	 			break;
				case 'count':
	 				$out .= '<td ' . $attributes . '>(' . self::get_count( $component ) . ')</td>';
	 			break;
			}
		}
		$out .= '</tr>';

		return $out;
	}

	public static function get_name( $component ) 
	{
		switch( $component->family  )
		{
			case 'core' :
				return 'wp';
			break;
			case 'theme' :
				return $component->info['current']['data']['headers']['Name'];
			break;
			case 'mu-plugin' :
			case 'plugin' :
				return basename( $component->info['data']['Name'] );
			break;
		}
		return '';
	}

	public static function get_desc( $component ) 
	{
		switch( $component->family  )
		{
			case 'core' :
				$desc = 'wp';
			break;
			case 'theme' :
				$desc = $component->info['current']['data']['headers']['Name'];
			break;
			case 'mu-plugin' :
			case 'plugin' :
				$desc = $component->info['data']['Name'];
			break;
		}
		return $desc;
	}

	public static function get_ver( $component ) 
	{
		switch( $component->family )
		{
			case 'core' :
				global $wp_version;
				$ver = $wp_version;
			break;
			case 'theme' :
				$ver = $component->info['current']['data']['headers']['Version'];
			break;
			case 'mu-plugin' :
			case 'plugin' :
				$ver = $component->info['data']['Version'];
			break;
		}

		$abbr = explode( '-', $ver );
		if ( is_array( $abbr) && ( count ( $abbr ) > 1 ) )
		{
			$ver  = trim( $abbr[0] );
			$abbr = trim( implode( '-', $abbr ) );
			$ver  = '<abbr title="' . $abbr . '">' . $ver . '</abbr>';                       
                }
		return $ver;
	}

	public static function get_event( $component ) 
	{
		$event = str_replace( 'gdpress_', '', $component->event );
		switch($event)
		{
			case 'debut' :
				$text = __( 'GDPress debut : %1$s', 'GDPress' );
			break;
			case 'finale' :
				$text = __( 'GDPress finale : %1$s', 'GDPress' );
			break;
			case 'snapshot' :
				$text = __( 'Changes detected : %1$s', 'GDPress' );
			break;
			case 'deactivated' :
				$text = __( 'Plugin deactivated : %1$s', 'GDPress' );
			break;
			case 'activated' :
				$text = __( 'Plugin activated : %1$s', 'GDPress' );
			break;
			default :
				$text = '%1$s';
			break;
		}
		return '<abbr title="' . sprintf( $text, wp_date( 'Y-m-d H:i:s', $component->request_time ) ) . '">' . $event . '</abbr>';
	}

	public static function get_count( $component ) 
	{
		global $wpdb;
		return $wpdb->get_var( $wpdb->prepare( "SELECT count(*) AS count FROM $wpdb->gdp_events WHERE 1=1 AND family = %s AND related = %s", $component->family, $component->related ) );
	}

	function admin_footer() 
	{
		require_once( GDP_ABSPATH . 'gp-admin/includes/templates/ropa_footer.php' );
	}
}
