<?php
class GDP_Archivist
{
	/**
	 * Unique class instance
	 *
	 * @static object
	 */
	private static $_instance = null;

	/**
	 * Event id
	 *
	 * @static integer
	 */
	private static $_event_id = 0;

	/**
	 * Event
	 *
	 * @static string
	 */
	private static $_event = null;

	/**
	 * Request id
	 *
	 * @static string
	 */
	public static $_request_id = null;

	/**
	 * Request
	 *
	 * @static string
	 */
	public static $_request = null;

	/**
	 * Action Name
	 *
	 * @static string
	 */
	private static $_action_name = null;

	/**
	 * resend
	 *
	 * @static bool
	 */
	private static $_resend = false;


	/**
	 * Constructor
	 *
	 * Constructor is private, use getInstance() static function instead
	 */

	private function __construct()
	{
		// entry points
		add_filter( 'parse_query', array( $this, 'found_posts' ), 10, 1 );
		add_action( 'check_admin_referer', array( $this, 'check_admin_referer' ), 1, 2 );
		add_action( 'login_form_confirmaction', array( $this, 'login_form_confirmaction' ), 1, 1 );
		add_filter( 'pre_delete_post', array( $this, 'pre_delete_post' ), 1, 3 );


		// detect admin actions
		new GDP_WP_Privacy();
	}

// requesting
	public function found_posts( $WP_Query )
	{
		// a query for The Query
		if ( ( ( isset( $WP_Query->query['post_type'], $WP_Query->query['post_name__in'][0], $WP_Query->query['title'], $WP_Query->query['post_status'], $WP_Query->query['fields'] ) ) ) && ( $WP_Query->query['post_type'] == 'user_request' && in_array( $WP_Query->query['post_name__in'][0], _wp_privacy_action_request_types() ) && is_email( $WP_Query->query['title'] ) && $WP_Query->query['post_status'] == array( 'request-pending', 'request-confirmed', ) && $WP_Query->query['fields'] == 'ids' ) )
		{
			self::$_action_name = $WP_Query->query['post_name__in'][0];
			
			$this->archive( 'requesting', true, array( 'metas' => array( 'action' => self::$_action_name, ), ), $WP_Query->query['title'] );

			add_filter( 'user_request_action_description', array( $this, 'user_request_action_description' ), 1, 2 );
		}
	}

// bulking (first hook on resend)
	public function check_admin_referer( $action, $result )
	{
		if ( $action != 'bulk-privacy_requests' ) return;
		self::$_resend = true;
		add_filter( 'user_request_action_email_headers', array( $this, 'user_request_action_email_headers' ), 1, 5 );
	}

// confirming
	public function login_form_confirmaction()
	{
		if ( isset( $_GET['gdprivacy'] ) ) new GDP_Download();

		if ( !( isset( $_GET['request_id'] )
		&&    ( $request = wp_get_user_request( $_GET['request_id'] ) ) ) ) return;

		self::$_request_id = $_GET['request_id'];
		self::$_action_name = $request->action_name;

		$this->archive( 'confirming' );

		add_action( 'user_request_action_confirmed', array( $this, 'user_request_action_confirmed' ), 1 );
	}

// removing
	public function pre_delete_post( $check, $post, $force_delete )
	{
		if ( ! ( $request = wp_get_user_request( $post->ID ) ) ) return;

		self::$_request     = $request;
		self::$_request_id  = self::$_request->ID;
		self::$_action_name = self::$_request->action_name;
		$this->archive( 'removing', false );

		add_action( 'deleted_post', array( $this, 'deleted_post' ), 1, 1 );

		return $check;
	}

	public function deleted_post( $postid )
	{
		$this->archive( 'removed', false );
	}

// something happening
	public function user_request_action_description( $description, $action_name )
	{
		if ( in_array( $action_name, _wp_privacy_action_request_types() ) )
		{
			if ( $action_name == self::$_action_name )
			{
				add_filter( 'user_request_action_email_headers',    array( $this, 'user_request_action_email_headers' ),    1, 5 );
				add_filter( 'user_request_confirmed_email_headers', array( $this, 'user_request_confirmed_email_headers' ), 1, 5 );
			}
		}

		return $description;
	}

// sending conf
	public function user_request_action_email_headers( $headers, $subject, $content, $request_id, $email_data )
	{
		self::$_request_id = $request_id;

		$event = 'sending_conflink';
		if ( self::$_resend ) $event = 're_' . $event;

		return $this->mail_event( $event, true, $headers, $subject, $content, $email_data );
	}

// confirmed
	public function user_request_action_confirmed( $request_id )
	{
		self::$_request_id = $request_id;

		$this->archive( 'confirmed' );

		add_filter( 'user_request_action_description', array( $this, 'user_request_action_description' ), 1, 2 );
	}

// admin notified
	public function user_request_confirmed_email_headers( $headers, $subject, $content, $request_id, $email_data )
	{
		self::$_request_id = $request_id;

		return $this->mail_event( 'notice_confirmed', false, $headers, $subject, $content, $email_data );
	}

// after admin approval, export sequence is going through exporters, firing events "export done" & "file created" and then sending the mail

	public function wp_privacy_personal_data_export_file( $request_id )
	{
		self::$_request_id = $request_id;

		self::$_event_id = $this->request_completed( 'exported', 'wp_privacy_personal_data_email_headers' );

		add_action( 'wp_privacy_personal_data_export_file_created', array( $this, 'wp_privacy_personal_data_export_file_created' ), 1, 5 );
	}

	public function wp_privacy_personal_data_export_file_created( $archive_pathname, $archive_url, $html_report_pathname, $request_id, $json_report_pathname )
	{
		foreach( array( 'html' => $html_report_pathname, 'json' => $json_report_pathname, ) as $format => $path )
		{
			if ( !is_file( $path ) ) continue;

			$content = @file_get_contents( $path );
			GDP_Event_meta::add( self::$_event_id, 'export_' . $format, $content, true ); 
		}
	}

	public function wp_privacy_personal_data_email_headers( $headers, $subject, $content, $request_id, $email_data )
	{
		self::$_request_id = $request_id;

		add_filter( 'wp_mail', array( $this, 'wp_mail' ), 1, 1 );

		return $this->mail_event( 'sending_exportlink', false, $headers, $subject, $content, $email_data );
	}

// after admin approval, erase sequence is going through erasers, sending the mail then firing event "erase done" 

	public function user_erasure_complete_email_headers( $headers, $subject, $content, $request_id, $email_data )
	{
		self::$_request_id = $request_id;

		add_action( 'wp_privacy_personal_data_erased', array( $this, 'wp_privacy_personal_data_erased' ), 15, 1 );

		return $this->mail_event( 'sending_erased', false, $headers, $subject, $content, $email_data );
	}

	public function wp_privacy_personal_data_erased( $request_id )
	{
		self::$_request_id = $request_id;

		$this->request_completed( 'erased', 'user_erasure_complete_email_headers' );
	}

//////////////

	public function request_completed( $event, $hook )
	{
		$id = $this->archive( $event, false );

		add_filter( $hook, array( $this, $hook ), 1, 5 );

		return $id;
	}

	public function mail_event( $event, $data_subject, $headers, $subject, $content, $email_data )
	{
		self::$_event = $event;

		$event_id = $this->archive( $event, $data_subject );

		$add['X-GDPress'] = sprintf( '%1$s, %2$s, %3$s', $event, self::$_request_id, $event_id );

		foreach( $add as $header => $value )
		{
			$header = sprintf( '%1$s: %2$s', trim( $header ), trim( $value ) );

			if ( is_string( $headers ) ) $headers = $header . "\n" . $headers;
			elseif ( is_array( $headers ) ) $headers = array_merge( array( $header ), $headers );
		}

		add_action( 'phpmailer_init', array( $this, 'phpmailer_init' ) );
		if ( class_exists( 'MailPress' ) ) add_action( 'MailPress_swift_message_built', array( $this, 'MailPress_swift_message_built' ), 1, 2 );

		return $headers;
	}

	public function wp_mail( $compact )
	{
		extract( $compact );

		if ( isset( self::$_request_id ) )
		{
			$export_file_url  = get_post_meta( self::$_request_id, '_export_file_url', true );
			$export_file_path = get_post_meta( self::$_request_id, '_export_file_path', true );

			$fname = basename( $export_file_path );
			$f = explode( '.', $fname );
			array_pop( $f );
			$f = implode( '.', $f );
			$f = str_replace( 'wp-personal-data-file-', '', $f );

			$f = $f . '.' . self::$_request_id;

			if ( $export_file_url )
			{
				$new_url = add_query_arg(
						array(
							'action'    => 'confirmaction',
							'gdprivacy' => $f,
						),
						wp_login_url()
				);

				$message = str_replace( $export_file_url, $new_url, $message );
			}
		}

		return compact( 'to', 'subject', 'message', 'headers', 'attachments' );
	}

	public function phpmailer_init( $mail ) 
	{
		$mail->preSend();
		$message = $mail->getSentMIMEMessage();

		$this->archive( self::$_event . '_phpmailer_presend', false, array( 'metas' => array( 'message' => $message, ), ) );

		remove_action( 'phpmailer_init', array( $this, 'phpmailer_init' ) );
	}

	public function MailPress_swift_message_built( $mail, $row ) 
	{
		$message = $mail->toString();

		$this->archive( self::$_event . '_swiftmailer_presend', false, array( 'metas' => array( 'message' => $message, 'mp_mail_id' => $row->id, ), ) );

		remove_action( 'MailPress_swift_message_built', array( $this, 'MailPress_swift_message_built' ) );
	}

//////////////

	public function archive( $event, $data_subject = true, $info = array(), $email = null ) 
	{
		return GDP_Event::insert( $event, $data_subject, $info, $email );
	}

	public static function getInstance()
	{
 		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}
}