<?php
class GDP_Event
{
	const object_type = 'event';

	const classes = array( 'GDP_WP_Privacy_eraser_gdpress', 'GDP_WP_Privacy_exporter_gdpress', 'GDP_Download', 'GDP_Archivist' );

	public static function get( $event, $output = OBJECT ) 
	{
		switch ( true )
		{
			case ( empty( $event ) ) :
				if ( isset( $GLOBALS['gdp_event'] ) ) 	$_event = & $GLOBALS['gdp_event'];
				else					return null;
			break;
			case ( is_object( $event ) ) :
				wp_cache_add( $event->id, $event, 'gdp_event' );
				$_event = $event;
			break;
			default :
				if ( isset( $GLOBALS['gdp_event'] ) && ( $GLOBALS['gdp_event']->id == $event ) ) 
				{
					$_event = & $GLOBALS['gdp_event'];
				} 
				elseif ( ! $_event = wp_cache_get( $event, 'gdp_event' ) ) 
				{
					global $wpdb;
					$_event = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $wpdb->gdp_events WHERE id = %d LIMIT 1", $event ) );
					if ( isset( $_event->info ) ) $_event->info = maybe_unserialize( $_event->info );
					if ( $_event ) wp_cache_add( $_event->id, $_event, 'gdp_event' );
				}
			break;
		}

		if ( $output == OBJECT ) {
			return $_event;
		} elseif ( $output == ARRAY_A ) {
			return get_object_vars( $_event );
		} elseif ( $output == ARRAY_N ) {
			return array_values( get_object_vars( $_event ) );
		} else {
			return $_event;
		}
	}

	public static function get_var( $var, $key_col, $key, $format = '%s' ) 
	{
		if (   $key_col == 'info'    ) $key = maybe_serialize( $key );
		if ( ( $key_col == 'related' ) && ( is_email( $key ) ) ) $key = md5( $key );

		global $wpdb;
		return $wpdb->get_var( $wpdb->prepare( "SELECT $var FROM $wpdb->gdp_events WHERE $key_col = $format LIMIT 1;", $key ) );
	}

	public static function get_last( $related, $family = '' )
	{
		if ( $family == '' ) $related = md5( $related );

		global $wpdb;
		$event = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $wpdb->gdp_events WHERE related = %s AND family = %s ORDER BY created DESC LIMIT 1", $related, $family ) );

		if ( $event ) $event->info = maybe_unserialize( $event->info );

		return $event;
	}

	public static function get_events( $related, $family = '' )
	{
		if ( $family == '' ) $related = md5( $related );

		global $wpdb;
		$events = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $wpdb->gdp_events WHERE related = %s AND family = %s ORDER BY created DESC;", $related, $family ) );

		$c = array();
		$x = $i = -1;
		foreach( $events as $k => $event )
		{
			if ( $i != $event->info )
			{
				$x++;
				$c[$x] = 0;	
				$i = $event->info;
				$events[$k]->parent = $x;
				continue;
			}

			$c[$x]++;
			$events[$k]->parent = $x;
		}
		$old_parent = false;
		foreach( $events as $k => $event )
		{
			$events[$k]->child = $c[$event->parent]--;

			if ( $events[$k]->parent !== $old_parent )
			{
				$old_parent = $events[$k]->parent;

				$events[$k]->info = maybe_unserialize( $events[$k]->info );

				switch( $events[$k]->family )
				{
					case 'core' :
						// component main file
						if ( isset( $events[$k]->info['data']['RoPA_time'] ) ) $events[$k]->info['data'] = array_merge( array( '>> File time' => sprintf( ' %1$s (%2$s)', wp_date( 'Y-m-d H:i:s', $events[$k]->info['data']['RoPA_time'] ), 'version.php' ), '_' => '' ), $events[$k]->info['data'] );

						if ( is_array( $events[$k]->info['readme'] ) )
						{
							if ( isset( $events[$k]->info['readme']['RoPA_time'] ) )
							{
								array_unshift( $events[$k]->info['readme'], sprintf( '>> File Time : %1$s (%2$s)', wp_date( 'Y-m-d H:i:s', $events[$k]->info['readme']['RoPA_time'] ), 'readme.html' ), '' );
							}
							else $events[$k]->info['readme']['No readme.html'] = array( __('> Unabled to access file /readme.html', 'GDPress') );
						}
						else $events[$k]->info['readme']['No readme.html'] = array( __('> Unabled to access file /readme.html', 'GDPress') );

					break;
					case 'theme' :
						foreach( array( 'current', 'parent' ) as $cp )
						{
							if ( isset( $events[$k]->info[$cp] ) )
							{
								// component $cp file
								$events[$k]->info[$cp]['data']   = self::format_data( $events[$k]->info[$cp]['data'], $events[$k] );
								$events[$k]->info[$cp]['readme'] = self::format_readme( $events[$k]->info[$cp]['readme'] );
							}
						}
					break;
					case 'mu-plugin' :
					case 'plugin' :
						// component main file
						if ( isset( $events[$k]->info['data']['RoPA_time'] ) ) $events[$k]->info['data'] = array_merge( array( '>> File time' => sprintf( ' %1$s (%2$s)', wp_date( 'Y-m-d H:i:s', $events[$k]->info['data']['RoPA_time'] ), $events[$k]->family ), '_' => '' ), $events[$k]->info['data'] );
						if (!isset( $events[$k]->info['data']['Version'] ) || ( $events[$k]->info['data']['Version'] == '' ) ) $events[$k]->info['data']['Version'] = "\"{$events[$k]->parent}-{$events[$k]->child}\"";
						$events[$k]->info['readme'] = self::format_readme( $events[$k]->info['readme'] );
					break;
				}

			}
			else
			{
				$events[$k]->info = NULL;
			}
		}

		return $events;
	}

	public static function format_data( $data, $event )
	{
		foreach( $data['headers'] as $k => $v ) { if ( is_array( $v ) ) unset( $data['headers'][$k] ); } // headers_sanitized, ...
		if ( isset( $data['RoPA_time'] ) )
		{
			$data['headers'] = array_merge( array( '>> File Time' => sprintf( ' %1$s (%2$s)', wp_date( 'Y-m-d H:i:s', $data['RoPA_time'] ) , 'stylesheet' ), '_' => '' ), $data['headers'] );
		}

		if ( !isset( $data['headers']['Version'] ) || ( $data['headers']['Version'] == '' ) ) $data['headers']['Version'] = "\"{$event->parent}-{$event->child}\"";

		return $data;
	}

	public static function format_readme( $readme )
	{
		$r = ( isset( $readme['name'] ) ) ? array( $readme['name'] => '', ) : array();
		$sort = array_merge( array( 'name' => '', 'RoPA_time' => '', ) ,  $r, array( 'Description' => '', 'Privacy' => '>> Empty', 'Copyright' => '', 'Frequently Asked Questions' => '', 'Changelog' => '', ) );

		if ( is_array( $readme ) )
		{
			if ( isset( $readme['RoPA_time'] ) )
			{
				array_unshift( $readme[$readme['name']], sprintf( '>> File Time : %1$s (%2$s)', wp_date( 'Y-m-d H:i:s', $readme['RoPA_time'] ), 'readme.txt' ), '' );
				$readme = array_merge( $sort, $readme );
			}
			else $readme['No readme.txt'] = array( __('> Unabled to access file /readme.txt', 'GDPress') );
		}
		else $readme['No readme.txt'] = array( __('> Unabled to access file /readme.txt', 'GDPress') );

		return $readme;
	}

	public static function format_debug_backtrace( $dbt )
	{
		$tab = '&rdsh; '; $tab_len = 2;

		$level = 0;
		$steps = array();

		krsort( $dbt );

		foreach ( $dbt as $k => $v )
		{
			$tabs = '';
			if ( $level )
			{
				$tabs = str_repeat( ' ', $tab_len * ( $level - 1 ) ) ;
				$tabs .= $tab;
			}

			$args = '';
			if ( isset( $v['args'] ) )
			{
				foreach( $v['args'] as $kk => $vv )
				{ 
					if ( $kk ) $args .= ', ';
					switch (true)
					{
						case is_bool( $vv ) :
							$args .= $vv ? 'true' : 'false';
						break;
						case is_numeric( $vv ) :
							$args .= $vv;
						break;
						case is_string( $vv ) :
							$vv = str_replace( array( "\r\n", "\n", '  ', '  ', ), ' ', $vv );
							if ( strlen( $vv ) > 50 ) $vv = substr( $vv, 0, 30 ) . '&#8230;';
							$args .= '\'' . $vv . '\'';
						break;
						case is_array( $vv ) :
							$args .= '[array(' . count( $vv ) . ')]';
						break;
						case is_object( $vv ) :
							$c = get_class( $vv );
							$args .= ( '__PHP_Incomplete_Class' == $c ) ? '[' . gettype( $vv ) . ']' : 'obj(' . get_class( $vv ) . ')';
						break;
						default :
							$args .= '[' . gettype( $vv ) . ']';
						break;
					}
				}
			}

			switch( true )
			{
				case ( isset( $v['class'] ) ) :
					$steps[] = $tabs . $v['class'] . $v['type'] . $v['function'] . '(' . $args . ')';
				break;
				case ( isset( $v['function'] ) ) :
					$steps[] = $tabs . $v['function'] . '(' . $args . ')';
				break;
				case ( isset( $v['file'] ) ) :
					$steps[] = $tabs . basename( $v['file'] );
				break;
			}

			$level++;
		}

		return implode( "\r\n", $steps );
	}

/// EVENT ///

	public static function update( $event, $data_subject = false, $info = array(), $related = NULL, $family = '' )
	{
		$_event = self::get_last( $related, $family );

		if ( ( $_event ) && ( $info == $_event->info ) ) return true;

		return self::insert( $event, $data_subject, $info, $related, $family );
	}

/// EVENT ///

	public static function insert( $event, $data_subject = false, $info = array(), $related = NULL, $family = '' )
	{
		if ( !isset( $related ) )
		{		
			foreach( self::classes as $class )
			{
				if ( class_exists( $class, false ) && isset( $class::$_request_id ) )
				{
					$request = wp_get_user_request( $class::$_request_id );
					if ( $request )
					{
						$info['metas']['request'] = $request;
						$info['metas']['action']  = $request->action_name;

						$related = $request->email;
						break;
					}
				}
			}
		}
		$metas = ( isset( $info['metas'] ) ) ? $info['metas'] : array();
		unset( $info['metas'] );
		if ( defined( 'GDP_DEBUG' ) && ( GDP_DEBUG ) ) $metas['debug_backtrace'] = self::format_debug_backtrace( debug_backtrace() );

		if ( empty( $family ) ) $related = md5( $related );
		$data_subject = ( $data_subject ) ? 1 : 0;


		$_TIME		 = time();
		$_URI	 	 = ( is_ssl() ? 'https://' : 'http://' ) . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

		$REMOTE_ADDR 	 = filter_input( INPUT_SERVER, 'REMOTE_ADDR' );
		$HTTP_USER_AGENT = filter_input( INPUT_SERVER, 'HTTP_USER_AGENT' );
		$WP_USER_AGENT   = apply_filters( 'http_headers_useragent', 'WordPress/' . get_bloginfo( 'version' ) . '; ' . get_bloginfo( 'url' ), $_URI );

		$agent = ( $HTTP_USER_AGENT == $WP_USER_AGENT ) ? '' : trim( strip_tags( $HTTP_USER_AGENT ) );


		$data = $format = array();

		$data['created'] 	= wp_date( 'Y-m-d H:i:s', $_TIME );	$format[] = '%s';
		$data['request_time'] 	= $_TIME;				$format[] = '%d';

		$data['event'] 		= $event; 				$format[] = '%s';
		$data['related'] 	= $related;				$format[] = '%s'; /* core => '', (plugin, theme) => id, '' => md5( $email ) */
		$data['family'] 	= $family;				$format[] = '%s'; /* core, plugin, theme, '' for human */
		$data['data_subject'] 	= $data_subject;			$format[] = '%d'; /* boolean */

		$data['wp_user_id'] 	= get_current_user_id();		$format[] = '%d';
		$data['ip']	 	= trim( $REMOTE_ADDR );			$format[] = '%s';
		$data['agent'] 		= $agent;				$format[] = '%s';
		$data['uri']		= $_URI;				$format[] = '%s';

		$data['info'] 		= maybe_serialize( $info );		$format[] = '%s'; /* any relevant information */

		global $wpdb;
		if ( !$wpdb->insert( $wpdb->gdp_events, $data, $format ) ) return false;

		$gdp_event_id = $wpdb->insert_id;

		foreach( $metas as $k => $v ) { GDP_Event_meta::add( $gdp_event_id, $k, $v, true ); }

		do_action( 'GDPress_insert_event', $gdp_event_id );

		return $gdp_event_id;
	}
}