<?php
class GDP_RoPA
{
	/**
	 * Unique class instance
	 *
	 * @static object
	 */
	private static $_instance = null;

	/**
	 * execution event (false for specific component event)
	 *
	 */
	public $event = false;


	const default_event = 'gdpress_snapshot';

	/**
	 * Constructor
	 *
	 * Constructor is private, use getInstance() static function instead
	 */
	private function __construct()
	{
	}

	public function do_core()
	{
		$info = array( 'data' => array(), 'readme' => array(), );

		global $wp_version;
		$fname = ABSPATH . WPINC . '/version.php';
		$info['data']['RoPA_time'] = filemtime( $fname );
		$info['data']['Version']   = $wp_version;

		$fname = ABSPATH . 'readme.html';
		$info['readme']['RoPA_time'] = filemtime( $fname );
		$info['readme']['html']      = file_get_contents( $fname );

		$this->archive( $info, '', 'core' );
	}

	public function do_themes()
	{
		$info = array( 'current' => array(), );

		$theme = wp_get_theme();

		$stylesheet = $theme->stylesheet;

		$info['current'] = $this->do_theme( $theme );

		if ( $parent = $theme->parent() )
		{
			$info['parent'] = $this->do_theme( $parent );
		}

		$this->archive( $info, $stylesheet, 'theme' );		
	}

	public function do_theme( $theme )
	{
		$dir = $theme->theme_root . '/' . $theme->stylesheet . '/';

		$info = array( 'data' => array(), 'readme' => array(), );

		$info['data']['Name'] = $theme->get( 'Name' );
		$fname = $dir . 'style.css';
		$info['data']['RoPA_time'] = filemtime( $fname );

		$_theme = (array) $theme;
		foreach( $_theme as $k => $v )
		{
			if ( strpos( $k, 'WP_Theme' ) == 1 ) $k = substr( $k, 10 ); 
			if( 'parent' == $k ) continue;

			if ( $k == 'headers' && is_array( $v ) )
			{
				$info['data']['headers'] = array();
				foreach( $v as $kk => $vv )
				{
					$vv = $theme->get( $kk );
					if ( is_array( $vv ) ) $vv = implode( ', ', $vv );
					$info['data']['headers'][$kk] = $vv;
				}
				$info['data']['headers'] = array_map( 'remove_accents', $info['data']['headers'] );
			}
			else
			{
				$info['data'][$k] = $v;
			}
		}

		$fname = $dir . 'readme.txt';
		$info['readme'] = self::parse_readme( $fname );

		return $info;
	}

	public function do_plugins()
	{
		$plugins = (array) get_plugins();
		foreach ( $plugins as $file => $data ) 
		{
			$this->do_plugin( $file, $data );
		}
	}

	public function do_plugin( $file, $data = false )
	{
		$info = array( 'data' => array(), 'readme' => array(), );

		if ( !$data )
		{
			$plugins = (array) get_plugins();
			$data = $plugins[$file] ?? false;
			if ( !$data ) return;
		}

		$is_mu  = ( is_plugin_active_for_network( $file ) );
		$dir    = ( $is_mu ) ? WPMU_PLUGIN_DIR : WP_PLUGIN_DIR;
		$dir    = ( is_file( $dir . '/' . $file ) ) ? $dir : WP_PLUGIN_DIR;
		$family = ( $is_mu ) ? 'mu-plugin' : 'plugin';

		$fname = $dir . '/' . $file;
		$info['data'] = array_merge( array( 'Name' => '', 'RoPA_time' => filemtime( $fname ) ), $data );

		$fname = $dir . '/' . dirname( $file ) . '/readme.txt';
		$info['readme'] = self::parse_readme( $fname );

		$this->archive( $info, $file, $family );
	}

	public function do_clean_up()
	{
		$options = get_option( GDPress::option_name_site_health );	
		if ( $options['remove-all'] )
		{
			wp_unschedule_hook( 'wp_site_health_scheduled_check' );
			wp_schedule_event( time() + 360 * DAY_IN_SECONDS, 'weekly', 'wp_site_health_scheduled_check' );
		}
	}

//// Really simple Readme.txt Parser ////

	public static function parse_readme( $fname )
	{
		if ( !is_file( $fname ) ) return NULL;

		$file = file( $fname );

		// removing bom
		$file[0] = trim( $file[0], "\\xef\\xbb\\xbf" );
		// removing leading and trailing spaces
		$file = array_map( 'trim', $file );
		// removing accentuated characters, not all readme.txt are from wp repositories ...
		$file = array_map( 'remove_accents', $file );

		$readme = array();
		$section = 'NULL';

		foreach( $file as $e )
		{
			if ( ( strpos( $e, '=== ' ) === 0 ) )
			{
				$section = str_replace( '=', '', $e );
				$section = trim( $section );
				$readme['name'] = $section;
				$readme['RoPA_time'] = filemtime( $fname );
			}
			elseif ( strpos( $e, '== ' ) === 0 )
			{
				$section = str_replace( '=', '', $e );
				$section = trim( $section );
				$section = ucwords( strtolower( $section ) );
			}
			else $readme[$section][] = $e;
		}
		return $readme;
	}

	public static function do_action( $component, $event, $id = false )
	{
		$actions = array( 'core', 'plugin', 'plugins', 'themes' );
		if ( !in_array( $component, $actions ) ) return;

		$action = 'do_' . $component;

		$ropa = self::getInstance( $event );

		$ropa->{$action}( $id );
	}

	public static function do_all( $event = false )
	{
		$ropa = self::getInstance( $event );

		$ropa->do_core();

		$ropa->do_themes();

		$ropa->do_plugins();

		$ropa->do_clean_up();
	}

//////////////

	public function archive( $info, $related, $family ) 
	{
		if ( !$this->event ) GDP_Event::update( self::default_event, false, $info, $related, $family );
		else GDP_Event::insert( $this->event, false, $info, $related, $family ); 	
	}

	public static function getInstance( $event = false )
	{
 		if ( is_null( self::$_instance ) )
		{
			self::$_instance = new self();
		}

		self::$_instance->event = $event;

		return self::$_instance;
	}
}