<?php
class GDP_WP_Admin_Bar_Menu
{
	private static $instance = null;

	function __construct( $wp_admin_bar )
	{
		if ( ( !current_user_can( GDPress::$const['export']['cap'] ) ) && ( !current_user_can( GDPress::$const['remove']['cap'] ) ) ) return;

		foreach( GDPress::$const as $k => $v )
		{
			if ( !isset( $v['post_name'] ) ) continue;
			$actions[$v['post_name']] = array( 
				'desc' => $v['desc'],
				'cap'  => $v['cap'],
				'mb'   => $v['menu']['mb'],
				'css'  => $v['menu']['css'],
				'url'  => $v['menu']['url'],
			);
		}

		foreach( $actions as $post_name => $action )
		{
			if ( ! current_user_can( $actions[$post_name]['cap'] ) )
			{
				unset( $actions[$post_name] );
				continue;
			}
			$actions[$post_name]['num_posts'] = self::get_num_posts( $post_name );
		}

		$icon   = '<span class="ab-icon"></span>';

		$subtitle = $srttitle = '';
		foreach( $actions as $post_name => $action )
		{
			if ( !empty( $subtitle ) ) $subtitle .= '/';
			if ( !empty( $srttitle ) ) $srttitle .= '/';
			if ( empty( $actions[$post_name]['num_posts'] ) ) continue;
			$subtitle .= '<span class="ab-label awaiting-mod pending-count vert_align_' . $actions[$post_name]['css'] . ' count-' . $actions[$post_name]['num_posts'] . '" aria-hidden="true">' . number_format_i18n(  $actions[$post_name]['num_posts']  ) . '</span>';
			$srttitle .= number_format_i18n( $actions[$post_name]['num_posts'] );
		}
		$title  = ( empty( $subtitle ) ) ? '' : '<span id="gdpr_ex_re">' . $subtitle . '</span>';
		$title .= ( empty( $srttitle ) ) ? '' : '<span class="screen-reader-text">' . sprintf( __( '%s awaiting' ) , $srttitle ) . '</span>';
 
		$wp_admin_bar->add_menu(
			array(
				'id'    => 'gdpress',
				'title' => $icon . $title,
				'href'  => $actions['export_personal_data']['url'] ?? $actions['remove_personal_data']['url'],
			)
		);

		foreach ( $actions as $post_name => $action )
		{
			extract( $action );

			if ( $num_posts ) $desc .= ' <span class="update-plugins ' . $num_posts . '"><span class="plugin-count" aria-hidden="true">' . $num_posts . '</span><span class="screen-reader-text">' . $num_posts . ' ' . $mb . '</span></span>';

			$wp_admin_bar->add_menu(
				array(
					'parent' => 'gdpress',
					'id'     => $post_name,
					'title'  => $desc,
					'href'   => ( $num_posts ) ? add_query_arg( array( 'filter-status' => 'request-confirmed' ), $url ) : $url,
				)
			);
		}
	}

	public static function get_num_posts( $post_name )
	{
		global $wpdb;
		return $wpdb->get_var( $wpdb->prepare( "SELECT COUNT( * ) AS num_posts FROM {$wpdb->posts} WHERE post_type = 'user_request' AND post_status = 'request-confirmed' AND post_name = %s;", $post_name ) );
	}

	public static function getInstance( $wp_admin_bar )
	{
		if ( !isset( self::$instance ) ) self::$instance = new self( $wp_admin_bar );
		return self::$instance;
	}
}