<?php
class GDP_WP_off_oEmbed
{
	function __construct()
	{
		add_action( 'plugins_loaded',   array( __CLASS__, 'plugins_loaded' ), 999 );
		add_filter( 'oembed_dataparse', array( __CLASS__, 'oembed_dataparse' ), 999, 3 );
	}

	public static function plugins_loaded() 
	{
		$options = get_option( GDPress::option_name_oembed);

		if ( !$options ) return;

		if ( $options['remove-all'] )
		{
			_wp_oembed_get_object()->providers = array();
			return;
		}

		$instance = GDP_WP_oEmbed_providers::getInstance();

		foreach( $options['providers'] as $domain => $settings )
		{
			if ( ! $settings['mode'] )
			{
				if ( !isset( $instance::$_providers[$domain] ) ) continue;
				foreach( $instance::$_providers[$domain] as $k => $v )
				{
					if ( !isset( $v['pattern'] ) ) continue;
					wp_oembed_remove_provider( $v['pattern'] );
				}
			}
		}
	}

	public static function oembed_dataparse( $return, $data, $url ) 
	{


		$options = get_option( GDPress::option_name_oembed);

		if ( $options['remove-all'] ) return $url;

		// which domain ?

		$instance = GDP_WP_oEmbed_providers::getInstance();

		foreach( $instance::$_providers as $domain => $_provider )
		{
			foreach( $_provider as $k => $v )
			{
				if ( !is_numeric( $k ) ) continue;
				if ( !is_array( $v ) ) continue;

				$providerurl = $v[0];
				$regex = $v[1];
				$matchmask = $v['pattern'];

				// Turn the asterisk-type provider URLs into regex.
				if ( ! $regex )
				{
					$matchmask = '#' . str_replace( '___wildcard___', '(.+)', preg_quote( str_replace( '*', '___wildcard___', $matchmask ), '#' ) ) . '#i';
					$matchmask = preg_replace( '|^#http\\\://|', '#https?\://', $matchmask );
				}

				if ( preg_match( $matchmask, $url ) )
				{
					break 2;
				}
			}
		}

		// what settings ?

		if ( !isset( $options['providers'][$domain]['mode'] ) ) return $return;

		$option = $options['providers'][$domain];
		$mode   = $option['mode'];

		// which format ? (best effort approach)
		$title = $_provider['name'] ?? uc_first( strtolower( $domain ) );
		$title = "[$title] ";
		$title .= ( isset( $data->title ) ) ? $data->title : '';

		switch( $mode )
		{
			case 9 :

				return $return;

			case 6 : // have an image ? no width / no height / do it with css
 
				if ( isset( $data->thumbnail_url ) )
					return sprintf( '<a href="%1$s" class="gdpress_oembed gdpress_oembed_%2$s"><img src="%3$s" class="gdpress_oembed gdpress_oembed_%2$s" alt="%4$s" title="%4$s" /></a>', 
						esc_url( $url ), 
						$_provider['id'], 
						esc_url( $data->thumbnail_url ), 
						esc_attr( $title )
					);

			case 3 : // have an anchor ?
				return sprintf( '<a href="%1$s" class="gdpress_oembed gdpress_oembed_%2$s" title="%3$s" />%4$s</a>', 
					esc_url( $url ), 
					$_provider['id'], 
					esc_attr( $title ),
					esc_attr( $data->title ?? $title )
				);
				return sprintf( '<a href="%1$s" class="gdpress_oembed gdpress_oembed_%2$s">%2$s</a>', esc_url( $url ), esc_attr( $data->title ?? '' ) );

			default :

				return $url;

			break;
		}

		return $url;
	}
}