<?php
class GDP_Widget extends WP_Widget
{
	function __construct()
	{
		$widget_ops  = array( 'classname' => 'widget_gdpress', 'description' => __( 'The privacy request form', 'GDPress' ) );
		$control_ops = array( 'width' => 400, 'height' => 300 );
		parent::__construct( 'GDPress', 'GDPress', $widget_ops, $control_ops );
	}

	function widget( $args, $instance ) 
	{
		extract( $args );
		$title = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'] );

		echo $before_widget;
		if ( !empty( $title ) )
		{
			echo $before_title . stripslashes( $title ) . $after_title;
		}

		$instance['widget_id'] = $args['widget_id'];
		self::widget_form( $instance ); 

		echo $after_widget;
	}

	function update( $new_instance, $old_instance ) 
	{
		$instance 				= $old_instance;
		$instance['title'] 		= strip_tags( $new_instance['title'] );
		$instance['label_submit'] 	= strip_tags( $new_instance['label_submit'] );
		return $instance;
	}

	function form( $instance ) 
	{
		$instance 	= wp_parse_args( ( array ) $instance, self::form_defaults() );
 		$title 	= ( isset( $instance['title'] ) ) ? strip_tags( $instance['title'] ) : '';
		$label_submit	= strip_tags( $instance['label_submit'] );
		$txtsubmgt	= strip_tags( $instance['txtsubmgt'] );
?>
<p>
	<label for="<?php echo $this->get_field_id( 'title' ); ?>">
		<?php _e( 'Title:' ); ?> 
		<input type="text" name="<?php echo $this->get_field_name( 'title' ); ?>" id="<?php echo $this->get_field_id( 'title' ); ?>" class="widefat" value="<?php echo esc_attr( $title ); ?>" />
	</label>
	<br /><br />
	<label for="<?php echo $this->get_field_id( 'label_submit' ); ?>">
		<?php _e( 'Button:' ); ?> 
		<input type="text" name="<?php echo $this->get_field_name( 'label_submit' ); ?>" id="<?php echo $this->get_field_id( 'label_submit' ); ?>" class="widefat" value="<?php echo esc_attr( $label_submit ); ?>" />
	</label>
</p>
<?php
	}

	public static function form_defaults( $options = array() ) 
	{
		$defaults = array(	'label_email' 	=> __( 'Your email', 'GDPress' ),
						'label_loading'	=> __( 'Processing&#8230;', 'GDPress' ), 
						'label_submit' 	=> __( 'Post Request', 'GDPress' ),

						'id_gdpress_field'	=> '_GDP_request',
		);

		$defaults = apply_filters( 'GDPress_form_defaults', $defaults );
		$options  = wp_parse_args( $options, $defaults );
		$options  = apply_filters( 'GDPress_form_options', $options );
		return $options;
	}

	public static function widget_form( $options = array() ) 
	{
		static $wp_head = false;

		$email = $widget_title = '';

		$options  = self::form_defaults( $options );

		$options_request = array (
			'id'		=> $options['id_gdpress_field'],
		);

		$id = $options['widget_id'];

		$post_ = filter_input_array( INPUT_POST );

		$user = wp_get_current_user();
		switch ( true )
		{
			case ( $user->ID ) :
				$email = $user->user_email;
			break;
			default :
				$requester = wp_get_current_commenter();
				$email     = $requester['comment_author_email'];
			break;
		}
		if ( '' == $email )
		{
			$email = $options['label_email'];
		}

		$is_valid = self::is_valid( $id ); /* if js is off on the browser, we have to insert the request */
		switch (true)
		{
			case is_wp_error( $is_valid ) :
				$message = $is_valid->get_error_message();
			break;
			case $is_valid :
				$message = __( 'Please check your email.', 'GDPress' );
			break;
			default :
				$message = false;
			break;
		}

		echo '<!-- start of code generated by GDPress ( ' . GDP_Version . ') -->';


		if ( !$wp_head )
		{
			$wp_head = true;

			// css
			$advanced_path = 'advanced/' . get_current_blog_id() . '/request-form';
			$root = GDP_UPL_ABSPATH . $advanced_path;
			echo '<style type="text/css">' . "\n"; include "$root/style.css"; echo "\n" . '</style>' . "\n";
			// jquery
			wp_enqueue_script( 'jquery' );
			// js
			$js = '/' . GDP_PATH . 'gp-includes/js/gdp_form.js';
			wp_register_script( 'gdpress_widget', $js );
			wp_localize_script( 'gdpress_widget', 'GDP_Widget', array( 
				'url' => admin_url( 'admin-ajax.php' )
			) );
			wp_enqueue_script( 'gdpress_widget' );
		}

		$imgloading = apply_filters( 'GDPress_form_imgloading', site_url() . '/' . GDP_PATH . 'gp-includes/images/loading.gif' );
?>
<div class="GDPress" id="_GDP_<?php echo $id; ?>">
	<div class="gdp-container">
<?php if ( $message ) : ?>
		<div class="gdp-message-no-js"><?php if ( $message ) echo $message; ?></div>
<?php else : ?>
		<div class="gdp-message"></div>
		<div class="gdp-loading"><img src="<?php echo $imgloading; ?>" alt="<?php echo esc_attr( $options['label_loading'] ); ?>" title="<?php echo esc_attr( $options['label_loading'] ); ?>" /><?php echo esc_html( $options['label_loading'] ); ?></div>
		<div class="gdp-formdiv">
			<form class="gdp-form" method="post">
				<input type="text"   name="email" class="GDPressFormEmail" size="25" value="<?php echo $email; ?>" onfocus="if( this.value=='<?php echo esc_js( $options['label_email'] ); ?>' ) this.value='';" onblur="if( this.value=='' ) this.value='<?php echo esc_js( $email ); ?>';"  />
				<br />
				<?php echo GDP_Request::get_select( $options_request ); ?>
				<br />
				<span style="display:none;"><input type="text" name="url" size="30" value="" /></span>
				<input type="hidden" name="gdp_action" 	value="widget_form" />
				<input type="hidden" name="action" 		value="gdpress_form" />
				<input type="hidden" name="id" 			value="<?php echo esc_attr( '_GDP_' . $id ); ?>" />
				<input type="submit" name="GDPress_submit" class="GDPressFormSubmit GDP_submit" value="<?php echo esc_attr( $options['label_submit'] ); ?>" />
			</form>
		</div>
<?php endif; ?>
	</div>
</div>
<?php


		echo '<!-- end of code generated by GDPress ( ' . GDP_Version . ') -->';


	}

	public static function handle_submission( $request_data )
	{
		if ( isset( $request_data['author'] ) )
		{
			return self::WP_Error();
		}

		if ( isset( $request_data['email'] ) && is_string( $request_data['email'] ) )
		{
			$request_data_email = trim( $request_data['email'] );
		}

		if ( ! ( isset( $request_data['url'] ) && empty( $request_data['url'] ) ) )
		{
			return self::WP_Error();
		}

		if ( isset( $request_data['comment'] ) )
		{
			return self::WP_Error();
		}

		if ( isset( $request_data['comment_parent'] ) )
		{
			return self::WP_Error();
		}

		if ( isset( $request_data['request'] ) && is_string( $request_data['request'] ) )
		{
			$request_data_request = trim( $request_data['request'] );
		}

		if ( ! is_email( $request_data_email ) )
		{
			return new WP_Error( 'require_valid_email', __( '<strong>ERROR</strong>: please enter a valid email address.' ), 200 );
		}

		if ( ! in_array( $request_data_request, _wp_privacy_action_request_types() ) )
		{
			return self::WP_Error();
		}

		$request_id = GDP_Request::new_( $request_data_email, $request_data_request, true ); 

		switch(true)
		{
			case ( is_wp_error( $request_id ) ) :
				return new WP_Error( 'invalid_request', $request_id->get_error_message(), 200 );
			break;
			case ( ! $request_id ) :
				return new WP_Error( 'aborted_request', esc_html__( 'We have a problem with your request. Please contact the administrator.', 'GDPress' ), 200 );
			break;
			default :
				return  wp_send_user_request( $request_id );
			break;
		}

		return new WP_Error( 'request_ok_mail_ko', esc_html__( 'We have a problem sending your confirmation mail. Please contact the administrator.', 'GDPress' ), 200 );
	}

	public static function is_valid( $widget_id )
	{
		$pst_ = filter_input_array( INPUT_POST );

		$widget_id = '_GDP_' . $widget_id ;

		$id        = $pst_['id'] ?? false;

		if ( $id != $widget_id )
		{
			return false;
		}

		return self::handle_submission( $pst_ );
	}
}